#!/usr/bin/env python

# poker-static.py - Generate a static representation of a poker game.
# Copyright (C) 2010 PyTom <pytom@bishoujo.us>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This program generates a static representation of a poker game, for
# use by AI players. The general idea is that, one we've shuffled the
# deck, a holdem game is predetermined. So we can statically precompute
# everything the game needs, store it in a file, and then load that
# file when it comes time to play the game.
#
# It first prints out the cards that will be dealt for the flop, turn,
# and river.
#
# For each player, it then prints out:
# - For the flop, turn, and river, the:
#   - pocket
#   - hand strength
#   - positive potential
#   - negative potential
# - The best hand the player can make.
#
# Finally, it prints out a list of lists of winners.

import os
import hand_ev # for py2exe.

# Lower the process priority ASAP.

import os.path
import platform
import sys
import random
import time

# Load pokereval.

path0 = os.path.abspath(os.path.dirname(sys.argv[0]))
path1 = os.path.join(path0, platform.system())

sys.path.insert(0, path0)
sys.path.insert(0, path1)

import pokereval
import hand_strength

pe = pokereval.PokerEval()

# Import the commands we'll be using into the local context.
poker_eval = pe.poker_eval
winners = pe.winners
strength_potential = hand_strength.strength_potential

def best_hand(hand):
    """
    Return the best hand that can be made with the cards in hand. We
    translate things into card strings.
    """

    hand = pe.string2card(hand)
    rv = pe.best_hand("hi", hand)

    for i in range(1, len(rv)):
        rv[i] = pe.card2string(rv[i])

    return rv

def main():

    fn = str(time.time()) + ".pok"
    f = file(fn + ".new", "w")
    
    deck = [ j + i for j in "AKQJT98765432" for i in "cshd" ]
    random.shuffle(deck)

    carditer = iter(deck)

    flop = [ carditer.next(), carditer.next(), carditer.next() ]
    turn = [ carditer.next(), ]
    river = [ carditer.next(), ]

    print >>f, "flop", " ".join(flop)
    print >>f, "turn", " ".join(turn)
    print >>f, "river", " ".join(river)

    pockets = [ ]
    
    for i in range(6):
        pocket = [ carditer.next(), carditer.next() ]
        pockets.append(pocket)
        
        print >>f, "pflop", pocket[0], pocket[1], "%f %f %f" % strength_potential(pocket, flop)
        print >>f, "pturn", pocket[0], pocket[1], "%f %f %f" % strength_potential(pocket, flop + turn)
        print >>f, "priver", pocket[0], pocket[1], "%f %f %f" % strength_potential(pocket, flop + turn + river)
        print >>f, "besthand", pocket[0], pocket[1], " ".join(best_hand(pocket + flop + turn + river))

    result = [ ]
    
    while pockets:

        wins = winners("holdem", board=flop + turn + river, pockets=pockets)["hi"]
        pocket_wins = [ pockets[i] for i in wins ]
        result.append(pocket_wins)

        for i in pocket_wins:
            pockets.remove(i)

    print >>f, "winners", result
    f.close()

    os.rename(fn + ".new", fn)
    
if __name__ == "__main__":
    main()


